@@ -1,6 +1,7 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright 2010 OpenStack LLC.
+# Copyright 2010 OpenStack Foundation
+# Copyright 2013 NTT corp.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -17,77 +18,89 @@
 
 """Implementation of an image service that uses Glance as the backend"""
 
+
 from __future__ import absolute_import
 
 import copy
 import itertools
 import random
+import shutil
 import sys
 import time
 import urlparse
 
-import glanceclient
 import glanceclient.exc
+from oslo.config import cfg
 
 from cinder import exception
-from cinder import flags
 from cinder.openstack.common import jsonutils
 from cinder.openstack.common import log as logging
 from cinder.openstack.common import timeutils
 
+glance_opts = [
+    cfg.ListOpt('allowed_direct_url_schemes',
+                default=[],
+                help='A list of url schemes that can be downloaded directly '
+                     'via the direct_url.  Currently supported schemes: '
+                     '[file].'),
+]
+CONF = cfg.CONF
+CONF.register_opts(glance_opts)
 
 LOG = logging.getLogger(__name__)
-FLAGS = flags.FLAGS
 
 
 def _parse_image_ref(image_href):
     """Parse an image href into composite parts.
 
     :param image_href: href of an image
-    :returns: a tuple of the form (image_id, host, port)
+    :returns: a tuple of the form (image_id, netloc, use_ssl)
     :raises ValueError
 
     """
     url = urlparse.urlparse(image_href)
-    port = url.port or 80
-    host = url.netloc.split(':', 1)[0]
+    netloc = url.netloc
     image_id = url.path.split('/')[-1]
     use_ssl = (url.scheme == 'https')
-    return (image_id, host, port, use_ssl)
+    return (image_id, netloc, use_ssl)
 
 
-def _create_glance_client(context, host, port, use_ssl,
-                          version=FLAGS.glance_api_version):
-    """Instantiate a new glanceclient.Client object"""
+def _create_glance_client(context, netloc, use_ssl,
+                          version=CONF.glance_api_version):
+    """Instantiate a new glanceclient.Client object."""
     if version is None:
-        version = FLAGS.glance_api_version
+        version = CONF.glance_api_version
+    params = {}
     if use_ssl:
         scheme = 'https'
+        # https specific params
+        params['insecure'] = CONF.glance_api_insecure
+        params['ssl_compression'] = CONF.glance_api_ssl_compression
     else:
         scheme = 'http'
-    params = {}
-    params['insecure'] = FLAGS.glance_api_insecure
-    if FLAGS.auth_strategy == 'keystone':
+    if CONF.auth_strategy == 'keystone':
         params['token'] = context.auth_token
-    endpoint = '%s://%s:%s' % (scheme, host, port)
+    if CONF.glance_request_timeout is not None:
+        params['timeout'] = CONF.glance_request_timeout
+    endpoint = '%s://%s' % (scheme, netloc)
     return glanceclient.Client(str(version), endpoint, **params)
 
 
 def get_api_servers():
-    """
-    Shuffle a list of FLAGS.glance_api_servers and return an iterator
+    """Return Iterable over shuffled api servers.
+
+    Shuffle a list of CONF.glance_api_servers and return an iterator
     that will cycle through the list, looping around to the beginning
     if necessary.
     """
     api_servers = []
-    for api_server in FLAGS.glance_api_servers:
+    for api_server in CONF.glance_api_servers:
         if '//' not in api_server:
             api_server = 'http://' + api_server
         url = urlparse.urlparse(api_server)
-        port = url.port or 80
-        host = url.netloc.split(':', 1)[0]
+        netloc = url.netloc
         use_ssl = (url.scheme == 'https')
-        api_servers.append((host, port, use_ssl))
+        api_servers.append((netloc, use_ssl))
     random.shuffle(api_servers)
     return itertools.cycle(api_servers)
 
@@ -95,40 +108,40 @@ def get_api_servers():
 class GlanceClientWrapper(object):
     """Glance client wrapper class that implements retries."""
 
-    def __init__(self, context=None, host=None, port=None, use_ssl=False,
+    def __init__(self, context=None, netloc=None, use_ssl=False,
                  version=None):
-        if host is not None:
+        if netloc is not None:
             self.client = self._create_static_client(context,
-                                                     host, port,
+                                                     netloc,
                                                      use_ssl, version)
         else:
             self.client = None
         self.api_servers = None
         self.version = version
 
-    def _create_static_client(self, context, host, port, use_ssl, version):
+    def _create_static_client(self, context, netloc, use_ssl, version):
         """Create a client that we'll use for every call."""
-        self.host = host
-        self.port = port
+        self.netloc = netloc
         self.use_ssl = use_ssl
         self.version = version
         return _create_glance_client(context,
-                                     self.host, self.port,
+                                     self.netloc,
                                      self.use_ssl, self.version)
 
     def _create_onetime_client(self, context, version):
         """Create a client that will be used for one call."""
         if self.api_servers is None:
             self.api_servers = get_api_servers()
-        self.host, self.port, self.use_ssl = self.api_servers.next()
+        self.netloc, self.use_ssl = self.api_servers.next()
         return _create_glance_client(context,
-                                     self.host, self.port,
+                                     self.netloc,
                                      self.use_ssl, version)
 
     def call(self, context, method, *args, **kwargs):
-        """
-        Call a glance client method.  If we get a connection error,
-        retry the request according to FLAGS.glance_num_retries.
+        """Call a glance client method.
+
+        If we get a connection error,
+        retry the request according to CONF.glance_num_retries.
         """
         version = self.version
         if version in kwargs:
@@ -137,7 +150,7 @@ class GlanceClientWrapper(object):
         retry_excs = (glanceclient.exc.ServiceUnavailable,
                       glanceclient.exc.InvalidEndpoint,
                       glanceclient.exc.CommunicationError)
-        num_attempts = 1 + FLAGS.glance_num_retries
+        num_attempts = 1 + CONF.glance_num_retries
 
         for attempt in xrange(1, num_attempts + 1):
             client = self.client or self._create_onetime_client(context,
@@ -145,19 +158,25 @@ class GlanceClientWrapper(object):
             try:
                 return getattr(client.images, method)(*args, **kwargs)
             except retry_excs as e:
-                host = self.host
-                port = self.port
+                netloc = self.netloc
                 extra = "retrying"
                 error_msg = _("Error contacting glance server "
-                              "'%(host)s:%(port)s' for '%(method)s', "
-                              "%(extra)s.")
+                              "'%(netloc)s' for '%(method)s', "
+                              "%(extra)s.") % {'netloc': netloc,
+                                               'method': method,
+                                               'extra': extra,
+                                               }
                 if attempt == num_attempts:
                     extra = 'done trying'
-                    LOG.exception(error_msg, locals())
-                    raise exception.GlanceConnectionFailed(host=host,
-                                                           port=port,
-                                                           reason=str(e))
-                LOG.exception(error_msg, locals())
+                    error_msg = _("Error contacting glance server "
+                                  "'%(netloc)s' for '%(method)s', "
+                                  "%(extra)s.") % {'netloc': netloc,
+                                                   'method': method,
+                                                   'extra': extra,
+                                                   }
+                    LOG.exception(error_msg)
+                    raise exception.GlanceConnectionFailed(reason=str(e))
+                LOG.exception(error_msg)
                 time.sleep(1)
 
 
@@ -212,9 +231,14 @@ class GlanceImageService(object):
 
     def get_location(self, context, image_id):
         """Returns the direct url representing the backend storage location,
-        or None if this attribute is not shown by Glance."""
+        or None if this attribute is not shown by Glance.
+        """
+        if CONF.glance_api_version == 1:
+            # image location not available in v1
+            return (None, None)
         try:
-            client = GlanceClientWrapper()
+            # direct_url is returned by v2 api
+            client = GlanceClientWrapper(version=2)
             image_meta = client.call(context, 'get', image_id)
         except Exception:
             _reraise_translated_image_exception(image_id)
@@ -222,17 +246,35 @@ class GlanceImageService(object):
         if not self._is_image_available(context, image_meta):
             raise exception.ImageNotFound(image_id=image_id)
 
-        return getattr(image_meta, 'direct_url', None)
+        # some glance stores like nfs only meta data
+        # is stored and returned as locations.
+        # so composite of two needs to be returned.
+        return (getattr(image_meta, 'direct_url', None),
+                getattr(image_meta, 'locations', None))
+
+    def download(self, context, image_id, data=None):
+        """Calls out to Glance for data and writes data."""
+        if 'file' in CONF.allowed_direct_url_schemes:
+            location = self.get_location(context, image_id)
+            o = urlparse.urlparse(location)
+            if o.scheme == "file":
+                with open(o.path, "r") as f:
+                    # a system call to cp could have significant performance
+                    # advantages, however we do not have the path to files at
+                    # this point in the abstraction.
+                    shutil.copyfileobj(f, data)
+                return
 
-    def download(self, context, image_id, data):
-        """Calls out to Glance for metadata and data and writes data."""
         try:
             image_chunks = self._client.call(context, 'data', image_id)
         except Exception:
             _reraise_translated_image_exception(image_id)
 
-        for chunk in image_chunks:
-            data.write(chunk)
+        if not data:
+            return image_chunks
+        else:
+            for chunk in image_chunks:
+                data.write(chunk)
 
     def create(self, context, image_meta, data=None):
         """Store the image data and return the new image object."""
@@ -250,15 +292,22 @@ class GlanceImageService(object):
                image_meta, data=None, purge_props=True):
         """Modify the given image with the new data."""
         image_meta = self._translate_to_glance(image_meta)
-        image_meta['purge_props'] = purge_props
+        #NOTE(dosaboy): see comment in bug 1210467
+        if CONF.glance_api_version == 1:
+            image_meta['purge_props'] = purge_props
         #NOTE(bcwaldon): id is not an editable field, but it is likely to be
         # passed in by calling code. Let's be nice and ignore it.
         image_meta.pop('id', None)
         if data:
             image_meta['data'] = data
         try:
-            image_meta = self._client.call(context, 'update', image_id,
-                                           **image_meta)
+            #NOTE(dosaboy): the v2 api separates update from upload
+            if data and CONF.glance_api_version > 1:
+                image_meta = self._client.call(context, 'upload', image_id,
+                                               image_meta['data'])
+            else:
+                image_meta = self._client.call(context, 'update', image_id,
+                                               **image_meta)
         except Exception:
             _reraise_translated_image_exception(image_id)
         else:
@@ -443,11 +492,9 @@ def get_remote_image_service(context, image_href):
         return image_service, image_href
 
     try:
-        (image_id, glance_host, glance_port, use_ssl) = \
-            _parse_image_ref(image_href)
+        (image_id, glance_netloc, use_ssl) = _parse_image_ref(image_href)
         glance_client = GlanceClientWrapper(context=context,
-                                            host=glance_host,
-                                            port=glance_port,
+                                            netloc=glance_netloc,
                                             use_ssl=use_ssl)
     except ValueError:
         raise exception.InvalidImageRef(image_href=image_href)
